package com.book.server.controller;

import com.book.dao.VO.SunVO;
import com.book.dao.polardb.entity.*;
import com.book.dao.utils.TimeUtil;
import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.config.SunTypeEnum;
import com.book.server.service.SunDataService;
import com.book.server.vo.SunNotifyRequest;
import com.book.server.vo.SunRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.websocket.server.PathParam;
import java.io.BufferedOutputStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * SUN
 * created in 2021/9/27
 * Project: book-store
 *
 * @author win7
 */
@RestController
@RequestMapping("/api/sun")
@Slf4j
public class SunDataController extends BaseController {
    @Autowired
    private SunDataService sunDataService;


    /**
     * getData
     *
     * @param type
     * @param request
     * @return
     */
    @PostMapping("/getData/{type}")
    public Result getData(@PathVariable("type") String type, @RequestBody SunRequest request) {
        SunTypeEnum sunTypeEnum = SunTypeEnum.valueOf(type);
        return sunDataService.requestData(request, sunTypeEnum);

    }

    /**
     * pullData
     *
     * @param request
     * @return
     */
    @PostMapping("/pullData")
    public String pushData(@RequestBody SunNotifyRequest request) {
        sunDataService.pullData(request);
        String s = "{\"code\":200,\"msg\":\"ok\"}";
        return s;

    }

    /**
     * 获取 渠道
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageChannel")
    public Result<PageResult<SunChannel>> pageSunChannel(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageSunChannel(sunVO));
    }

    /**
     * 获取 订单明细表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageOrder")
    public Result<PageResult<SunOrder>> pageOrder(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageSunOrder(sunVO));
    }

    /**
     * 获取 订单统计表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageOrderCollect")
    public Result<PageResult<SunOrderCollect>> pageOrderCollect(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageOrderCollect(sunVO));
    }

    /**
     * 获取 用户统计表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageUserCollect")
    public Result<PageResult<SunUserCollect>> pageUserCollect(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageUserCollect(sunVO));
    }

    /**
     * 订单统计-重置某天
     *
     * @param date
     * @return
     */
    @GetMapping("/orderStaticByDate-F891088E-7A75-4F4B-AEB4-A9D1697F5AD3")
    public Result orderStaticByDate(String date) {
        sunDataService.orderStaticDate(date);
        return Result.success();
    }

    /**
     * 用户统计-重置某天
     *
     * @return
     */
    @GetMapping("/userStaticByDate-F891088E-7A75-4F4B-AEB4-A9D1697F5AB3")
    public Result userStaticByDate(String startDate, String endDate) {
        sunDataService.userStaticYesterday(startDate, endDate);
        return Result.success();
    }

    /**
     * 订单统计-获取今天
     *
     * @return
     */
    @GetMapping("/getTodayOrderStatic")
    public Result getTodayOrderStatic() {
        return Result.success(sunDataService.getTodayOrderStatic());
    }

    /**
     * 用户统计-获取今天
     *
     * @return
     */
    @GetMapping("/getTodayUserStatic")
    public Result getTodayUserStatic() {
        return Result.success(sunDataService.getTodayUserStatic());
    }

    /**
     * 订单统计-今天按渠道
     *
     * @return
     */
    @GetMapping("/getTodayOrderStaticByChannel")
    public Result getTodayOrderStaticByChannel() {
        return Result.success(sunDataService.getTodayOrderStaticByChannel());
    }

    /**
     * 用户统计-今天按渠道
     *
     * @return
     */
    @GetMapping("/getTodayUserStaticByChannel")
    public Result getTodayUserStaticByChannel() {
        return Result.success(sunDataService.getTodayUserStaticByChannel());
    }

    public void export(HttpServletResponse response, String fileName, List<String> list) throws UnsupportedEncodingException {

        response.setContentType("text/plain");
        response.setHeader("Content-Disposition", "attachment; filename=" + java.net.URLEncoder.encode(fileName, "UTF-8"));

        BufferedOutputStream buff = null;
        StringBuilder write = new StringBuilder();
        String enter = "\r\n";
        ServletOutputStream outSTr = null;
        try {
            outSTr = response.getOutputStream();
            buff = new BufferedOutputStream(outSTr);
            // 把内容写入文件
            for (String s : list) {
                write.append(s);
                write.append(enter);
            }
            buff.write(write.toString().getBytes(StandardCharsets.UTF_8));
            buff.flush();
            buff.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                assert buff != null;
                buff.close();
                outSTr.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 订单统计-导出当天
     *
     * @param response
     * @throws UnsupportedEncodingException
     */
    @GetMapping("/exportTodayOrderStaticByChannel")
    public void exportTodayOrderStaticByChannel(HttpServletResponse response) throws UnsupportedEncodingException {
        String fileName = TimeUtil.getTodayStr() + "-" + UUID.randomUUID() + ".csv";
        ArrayList<String> strings = new ArrayList<>();

        List<SunStaticOrderDayChannel> list = sunDataService.getTodayOrderStaticByChannel();
        strings.add("日期,渠道 Id,渠道名称,活跃用户数,充值用户数,充值金额,充值笔数,充值人均");
        for (SunStaticOrderDayChannel text : list) {
            strings.add(text.getStaticDate() + "," + text.getChannelId() + "," + text.getChannelName() + "," + text.getActiveCount() + "," + text.getRechargeUserCount() + "," + text.getRechargeSum() + "," + text.getRechargeCount() + "," + text.getRechargeAvg());
        }
        export(response, fileName, strings);
    }

    /**
     * 用户统计-导出当天
     *
     * @param response
     * @throws UnsupportedEncodingException
     */
    @GetMapping("/exportTodayUserStaticByChannel")
    public void exportTodayUserStaticByChannel(HttpServletResponse response) throws UnsupportedEncodingException {
        String fileName = TimeUtil.getTodayStr() + "-" + UUID.randomUUID() + ".csv";
        ArrayList<String> strings = new ArrayList<>();

        List<SunStaticUserDayChannel> list = sunDataService.getTodayUserStaticByChannel();
        strings.add("日期,渠道 Id,渠道名称,新增用户数,新增用户的充值人数,新增用户充值金额（元）,新增用户充值笔数,新增用户充值人均,新增付费率,单粉价");
        for (SunStaticUserDayChannel text : list) {
            strings.add(text.getStaticDate() + ","
                    + text.getChannelId() + ","
                    + text.getChannelName() + ","
                    + text.getNewAllUser() + ","
                    + text.getNewRechargeUser() + ","
                    + text.getNewRechargeSum() + ","
                    + text.getNewRechargeCount() + ","
                    + text.getRechargeAvg() + ","
                    + text.getRechargeRate() + ","
                    + text.getPrice());
        }
        export(response, fileName, strings);
    }

    /**
     * 订单统计-导出历史
     *
     * @param response
     * @throws UnsupportedEncodingException
     */
    @GetMapping("/exportOrderStaticAll")
    public void exportTodayOrderStatic(HttpServletResponse response) throws UnsupportedEncodingException {

        String fileName = TimeUtil.getTodayStr() + UUID.randomUUID() + ".csv";
        ArrayList<String> strings = new ArrayList<>();

        List<SunStaticOrderDay> list = sunDataService.getOrderStaticsAll();
        strings.add("日期,活跃用户数,充值用户数,充值金额,充值笔数,充值人均");
        for (SunStaticOrderDay text : list) {
            strings.add(text.getStaticDate() + "," + text.getActiveCount() + "," + text.getRechargeUserCount() + "," + text.getRechargeSum() + "," + text.getRechargeCount() + "," + text.getRechargeAvg());
        }
        export(response, fileName, strings);
    }

    /**
     * 用户统计-导出历史
     *
     * @param response
     * @throws UnsupportedEncodingException
     */
    @GetMapping("/exportUserStaticAll")
    public void exportUserStaticAll(HttpServletResponse response) throws UnsupportedEncodingException {

        String fileName = TimeUtil.getTodayStr() + UUID.randomUUID() + ".csv";
        ArrayList<String> strings = new ArrayList<>();

        List<SunStaticUserDay> list = sunDataService.getUserStaticsAll();
        strings.add("日期,新增用户数,新增用户的充值人数,新增用户充值金额（元）,新增用户充值笔数,新增用户充值人均,新增付费率,单粉价");
        for (SunStaticUserDay text : list) {
            strings.add(text.getStaticDate() + ","
                    + text.getNewAllUser() + ","
                    + text.getNewRechargeUser() + ","
                    + text.getNewRechargeSum() + ","
                    + text.getNewRechargeCount() + ","
                    + text.getRechargeAvg() + ","
                    + text.getRechargeRate() + ","
                    + text.getPrice());
        }
        export(response, fileName, strings);
    }

    /**
     * 订单统计-昨天
     *
     * @return
     */
    @GetMapping("/getYesterdayOrderStatic")
    public Result getYesterdayOrderStatic() {
        return Result.success(sunDataService.getYesterdayOrderStatic());
    }

    /**
     * 用户统计-昨天
     *
     * @return
     */
    @GetMapping("/getYesterdayUserStatic")
    public Result getYesterdayUserStatic() {
        return Result.success(sunDataService.getYesterdayUserStatic());
    }

    /**
     * 订单统计-历史
     *
     * @return
     */
    @GetMapping("/getHistoryOrderStatic")
    public Result getHistoryOrderStatic() {
        return Result.success(sunDataService.getHistoryOrderStatic());
    }

    /**
     * 用户统计-历史
     *
     * @return
     */
    @GetMapping("/getHistoryUserStatic")
    public Result getHistoryUserStatic() {
        return Result.success(sunDataService.getHistoryUserStatic());
    }

    /**
     * 订单统计-按天排序
     *
     * @return
     */
    @GetMapping("/getOrderStatics")
    public Result getOrderStatics(int page, int size) {
        return Result.success(sunDataService.getOrderStatics(page, size));
    }

    /**
     * 用户统计-按天排序
     *
     * @return
     */
    @GetMapping("/getUserStatics")
    public Result getUserStatics(int page, int size) {
        return Result.success(sunDataService.getUserStatics(page, size));
    }

    /**
     * 订单统计-按天查询不同渠道
     *
     * @return
     */
    @GetMapping("/getOrderStaticByDateAndChannel")
    public Result getOrderStaticByDateAndChannel(String date) {
        return Result.success(sunDataService.getOrderStaticByDateAndChannel(date));
    }

    /**
     * 用户统计-按天查询不同渠道
     *
     * @return
     */
    @GetMapping("/getUserStaticByDateAndChannel")
    public Result getUserStaticByDateAndChannel(String date) {
        return Result.success(sunDataService.getUserStaticByDateAndChannel(date));
    }


    public static void main(String[] args) {
        System.out.println(UUID.randomUUID().toString());
    }

}
